package com.mwq.mwing;

import java.awt.BorderLayout;
import java.awt.Color;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JViewport;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;

public class FixedColumnTablePanel extends JPanel {

	private MTable fixedColumnTable;

	private FixedColumnTableModel fixedColumnTableModel;

	private MTable floatingColumnTable;

	private FloatingColumnTableModel floatingColumnTableModel;

	private Vector<String> tableColumnV;

	private Vector<Vector<Object>> tableValueV;

	private int fixedColumn = 2;

	public FixedColumnTablePanel(Vector<String> tableColumnV,
			Vector<Vector<Object>> tableValueV, int fixedColumn) {
		super();
		setLayout(new BorderLayout());

		this.tableColumnV = tableColumnV;
		this.tableValueV = tableValueV;
		this.fixedColumn = fixedColumn;

		fixedColumnTableModel = new FixedColumnTableModel();// ̶бģͶ

		fixedColumnTable = new MTable(fixedColumnTableModel);// ̶б
		ListSelectionModel fixed = fixedColumnTable.getSelectionModel();// ѡģͶ
		fixed.addListSelectionListener(new MListSelectionListener(true));// бѡе¼

		floatingColumnTableModel = new FloatingColumnTableModel();// ƶбģͶ

		floatingColumnTable = new MTable(floatingColumnTableModel);// ƶб
		ListSelectionModel floating = floatingColumnTable.getSelectionModel();// ѡģͶ
		floating.addListSelectionListener(new MListSelectionListener(false));// бѡе¼

		JScrollPane scrollPane = new JScrollPane();// һ
		scrollPane.setCorner(JScrollPane.UPPER_LEFT_CORNER, fixedColumnTable
				.getTableHeader()); // ̶бͷŵϷ

		JViewport viewport = new JViewport();// һʾϢӿڶ
		viewport.setView(fixedColumnTable);// ̶бӵӿ
		viewport.setPreferredSize(fixedColumnTable.getPreferredSize());// ӿڵѡСΪ̶бѡС
		scrollPane.setRowHeaderView(viewport);// ӿӵıӿ

		scrollPane.setViewportView(floatingColumnTable);// ƶӵĬӿ

		add(scrollPane, BorderLayout.CENTER);
		//
	}

	class MTable extends JTable {

		public MTable(AbstractTableModel tableModel) {
			super(tableModel);
			setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		}

		// Ϣ
		@Override
		public JTableHeader getTableHeader() {
			// ñͷ
			JTableHeader tableHeader = super.getTableHeader();
			tableHeader.setReorderingAllowed(false);// ñв
			// ñͷĵԪ
			DefaultTableCellRenderer defaultRenderer = (DefaultTableCellRenderer) tableHeader
					.getDefaultRenderer();
			// õԪݣʾ
			defaultRenderer
					.setHorizontalAlignment(DefaultTableCellRenderer.CENTER);
			return tableHeader;
		}

		// ֵʾ
		@Override
		public TableCellRenderer getDefaultRenderer(Class<?> columnClass) {
			DefaultTableCellRenderer defaultRenderer = (DefaultTableCellRenderer) super
					.getDefaultRenderer(columnClass);
			defaultRenderer
					.setHorizontalAlignment(DefaultTableCellRenderer.CENTER);
			return defaultRenderer;
		}

		// ֻɵѡ
		@Override
		public ListSelectionModel getSelectionModel() {
			ListSelectionModel selectionModel = super.getSelectionModel();
			selectionModel
					.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			return selectionModel;
		}

		// ñѡ
		@Override
		public void setRowSelectionInterval(int fromRow, int toRow) {// عķ
			super.setRowSelectionInterval(fromRow, toRow);
		}

		// ñΨһѡ
		public void setRowSelectionInterval(int row) {// ͨʵԼķ
			setRowSelectionInterval(row, row);
		}

	}

	class FixedColumnTableModel extends AbstractTableModel {

		public int getColumnCount() {// ع̶е
			return fixedColumn;
		}

		public int getRowCount() {// 
			return tableValueV.size();
		}

		public Object getValueAt(int rowIndex, int columnIndex) {// ָԪֵ
			return tableValueV.get(rowIndex).get(columnIndex);
		}

		@Override
		public String getColumnName(int columnIndex) {// ָе
			return tableColumnV.get(columnIndex);
		}

	}

	class FloatingColumnTableModel extends AbstractTableModel {

		public int getColumnCount() {// ؿƶе
			return tableColumnV.size() - fixedColumn;// Ҫ۳̶е
		}

		public int getRowCount() {// 
			return tableValueV.size();
		}

		public Object getValueAt(int rowIndex, int columnIndex) {// ָԪֵ
			return tableValueV.get(rowIndex).get(columnIndex + fixedColumn);// ҪΪϹ̶е
		}

		@Override
		public String getColumnName(int columnIndex) {// ָе
			return tableColumnV.get(columnIndex + fixedColumn);// ҪΪϹ̶е
		}

	}

	class MListSelectionListener implements ListSelectionListener {
		boolean isFixedColumnTable = true; // Ĭѡй̶бед

		public MListSelectionListener(boolean isFixedColumnTable) {
			this.isFixedColumnTable = isFixedColumnTable;
		}

		public void valueChanged(ListSelectionEvent e) {
			if (isFixedColumnTable) { // ѡй̶бед
				int selectedRow = fixedColumnTable.getSelectedRow(); // ù̶беѡ
				floatingColumnTable.setRowSelectionInterval(selectedRow); // ͬʱѡпƶбеѡ
			} else { // ѡпƶбед
				int selectedRow = floatingColumnTable.getSelectedRow(); // ÿƶбеѡ
				fixedColumnTable.setRowSelectionInterval(selectedRow); // ͬʱѡй̶беѡ
			}
		}
	}

}
